"""Build webapps using IPywidgets"""
__version__ = "0.4.1"
# isort: skip_file

from . import util

# flake8: noqa: F402
from .datatypes import *
from .hooks import *
from .memoize import memoize
from .components import *

github_url = "https://github.com/widgetti/solara"
git_branch = "master"
